@@ -49,17 +49,13 @@
 
 package com.openexchange.push.imapidle.locking;
 
-import java.util.LinkedHashSet;
-import java.util.Map;
 import java.util.Set;
-import java.util.concurrent.CancellationException;
 import java.util.concurrent.ExecutionException;
-import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
 import com.google.common.cache.Cache;
 import com.google.common.cache.CacheBuilder;
-import com.hazelcast.core.Cluster;
 import com.hazelcast.core.HazelcastInstance;
+import com.hazelcast.core.Hazelcasts;
 import com.hazelcast.core.IExecutorService;
 import com.hazelcast.core.Member;
 import com.openexchange.server.ServiceLookup;
@@ -74,8 +70,6 @@ import com.openexchange.sessionstorage.hazelcast.serialization.PortableSessionEx
  */
 public abstract class AbstractImapIdleClusterLock implements ImapIdleClusterLock {
 
-    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(AbstractImapIdleClusterLock.class);
-
     /** The time-to-live in cache; cluster lock timeout minus 5 minutes */
     private static final long CACHE_EXPIRATION = ImapIdleClusterLock.TIMEOUT_MILLIS - 300000L;
 
@@ -284,74 +278,34 @@ public abstract class AbstractImapIdleClusterLock implements ImapIdleClusterLock
     }
 
     private boolean checkRemoteMembersIfSessionExists(String sessionId, HazelcastInstance hzInstance) {
-        // Check in cluster
-        Cluster cluster = hzInstance.getCluster();
-
-        // Get local member
-        Member localMember = cluster.getLocalMember();
-
         // Determine other cluster members
-        Set<Member> otherMembers = getOtherMembers(cluster.getMembers(), localMember);
+        Set<Member> otherMembers = Hazelcasts.getRemoteMembers(hzInstance);
 
         if (!otherMembers.isEmpty()) {
-            IExecutorService executor = hzInstance.getExecutorService("default");
-            Map<Member, Future<Boolean>> futureMap = executor.submitToMembers(new PortableSessionExistenceCheck(sessionId), otherMembers);
-            for (Map.Entry<Member, Future<Boolean>> entry : futureMap.entrySet()) {
-                Future<Boolean> future = entry.getValue();
-                // Check Future's return value
-                int retryCount = 3;
-                while (retryCount-- > 0) {
-                    try {
-                        boolean exists = future.get().booleanValue();
-                        retryCount = 0;
-                        if (exists) {
-                            return true;
-                        }
-                    } catch (InterruptedException e) {
-                        // Interrupted - Keep interrupted state
-                        Thread.currentThread().interrupt();
-                    } catch (CancellationException e) {
-                        // Canceled
-                        retryCount = 0;
-                    } catch (ExecutionException e) {
-                        Throwable cause = e.getCause();
+            Hazelcasts.Filter<Boolean, Boolean> filter = new Hazelcasts.Filter<Boolean, Boolean>() {
 
-                        // Check for Hazelcast timeout
-                        if (!(cause instanceof com.hazelcast.core.OperationTimeoutException)) {
-                            if (cause instanceof RuntimeException) {
-                                throw ((RuntimeException) cause);
-                            }
-                            if (cause instanceof Error) {
-                                throw (Error) cause;
-                            }
-                            throw new IllegalStateException("Not unchecked", cause);
-                        }
-
-                        // Timeout while awaiting remote result
-                        if (retryCount <= 0) {
-                            // No further retry
-                            cancelFutureSafe(future);
-                        }
-                    }
+                @Override
+                public Boolean accept(Boolean result) {
+                    return result.booleanValue() ? Boolean.TRUE : null;
                 }
+            };
+            IExecutorService executor = hzInstance.getExecutorService("default");
+            try {
+                Boolean exists = Hazelcasts.executeByMembersAndFilter(new PortableSessionExistenceCheck(sessionId), otherMembers, executor, filter);
+                return null != exists && exists.booleanValue();
+            } catch (ExecutionException e) {
+                Throwable cause = e.getCause();
+                if (cause instanceof RuntimeException) {
+                    throw ((RuntimeException) cause);
+                }
+                if (cause instanceof Error) {
+                    throw (Error) cause;
+                }
+                throw new IllegalStateException("Not unchecked", cause);
             }
         }
 
         return false;
     }
 
-    static Set<Member> getOtherMembers(Set<Member> allMembers, Member localMember) {
-        Set<Member> otherMembers = new LinkedHashSet<Member>(allMembers);
-        if (!otherMembers.remove(localMember)) {
-            LOG.warn("Couldn't remove local member from cluster members.");
-        }
-        return otherMembers;
-    }
-
-    static void cancelFutureSafe(Future<Boolean> future) {
-        if (null != future) {
-            try { future.cancel(true); } catch (Exception e) {/*Ignore*/}
-        }
-    }
-
 }
